#!/usr/bin/env python

'''
Sample show VideoCapture via v4l2 interface for PHYTEC phyCAM-P and phyCAM-S+ series.
Keys:
    q    - exit
'''

# Python 2/3 compatibility
import numpy as np
import cv2
import subprocess
import sys
import os

print ("")
print ("choose you camera")
print ("=================")
print ("1 = VM-010-BW,  Resolution (752 x 480)")
print ("2 = VM-010-COL, Resolution (752 x 480)")
print ("3 = VM-011-BW,  Resolution (640 x 480) ROI")
print ("4 = VM-011-COL, Resolution (640 x 480) ROI")
print ("5 = VM-016-BW,  Resolution (640 x 480) ROI")
print ("6 = VM-016-COL, Resolution (640 x 480) ROI")
camera_version = input("Your select = ")

# for other resolutions change the "grab_res" and "cam_offset" parameter
if camera_version == "1":
	cam_typ = "'mt9v032 0-0048'"; grab_res = "752x480"; cam_col_fmt = "Y8"; cam_offset = "1,5"; cam_width = 752; cam_height = 480;
	v4l2_ctl_param = "row_noise_correction=1"
elif camera_version == "2":
	cam_typ = "'mt9v032 0-0048'"; grab_res = "752x480"; cam_col_fmt = "SGRBG8_1X8"; cam_offset = "1,5"; cam_width = 752; cam_height = 480;
	v4l2_ctl_param = "row_noise_correction=1"
elif camera_version == "3":
	cam_typ = "'mt9p031 0-0048'"; grab_res = "640x480"; cam_col_fmt = "Y8"; cam_offset = "976,732"; cam_width = 640; cam_height = 480;
	v4l2_ctl_param = "exposure=720,gain=32"
elif camera_version == "4":
	cam_typ = "'mt9p031 0-0048'"; grab_res = "640x480"; cam_col_fmt = "SGRBG8_1X8"; cam_offset = "976,732"; cam_width = 640; cam_height = 480;
	v4l2_ctl_param = "exposure=720,gain=32"
elif camera_version == "5":
	cam_typ = "'ar0144 0-0010'"; grab_res = "640x480"; cam_col_fmt = "Y8"; cam_offset = "320,160"; cam_width = 640; cam_height = 480;
	v4l2_ctl_param = "auto_exposure_max=1600"
elif camera_version == "6":
	cam_typ = "'ar0144 0-0010'"; grab_res = "640x480"; cam_col_fmt = "SGRBG8_1X8"; cam_offset = "320,160"; cam_width = 640; cam_height = 480;
	v4l2_ctl_param = "auto_exposure_max=1600,vertical_blanking=858,autogain_digital=0,digital_gain_red=1200,digital_gain_blue=1200"
else:
	print ("wrong number")
	sys.exit(0)

# set display
os.environ["DISPLAY"] = ':0.0'


# read /dev/video[x] camera at IPU1_CSI0
ret = subprocess.run(["media-ctl", "-e", "ipu1_csi0 capture"], stdout=subprocess.PIPE)
IPU1_CSI0_DEVICE = ret.stdout.decode()					# .decode -> convert returnvalue from byte in string
IPU1_CSI0_DEVICE = IPU1_CSI0_DEVICE[:-1]				# cut the "\n" at the end
IPU1_CSI0_DEVICE_NUMBER = int(IPU1_CSI0_DEVICE[10:].strip())	# .strip() -> cut the first 10 characters
print ("")
print (" camera device for CSI1 port = ", IPU1_CSI0_DEVICE)
print (" camera device_number for CSI1 port = ", IPU1_CSI0_DEVICE_NUMBER)


# set video path, camera resolution and ROI position via media-ctl
print ("")
print (" configure IPU1_CSI0_DEVICE (camera_1 port) with v4l2_control")
print (" Note: For use CSI2_Device (camera_2 port) or 2 cameras or other")
print (" resolutions, see the scripts in path ...\\gstreamer_examples\\...")
print (" ===============================================================")
subprocess.run(["media-ctl", "-r"])
subprocess.run(["media-ctl", "-l", cam_typ+":0->'ipu1_csi0_mux':1[1]"])
subprocess.run(["media-ctl", "-l", "'ipu1_csi0_mux':2->'ipu1_csi0':0[1]"])
subprocess.run(["media-ctl", "-l", "'ipu1_csi0':2->'ipu1_csi0 capture':0[1]"])

subprocess.run(["media-ctl", "-V", cam_typ+":0 [fmt:"+cam_col_fmt+"/"+grab_res+" ("+cam_offset+")/"+grab_res+"]"])
subprocess.run(["media-ctl", "-V", "'ipu1_csi0_mux':2 [fmt:"+cam_col_fmt+"/"+grab_res+"]"])
subprocess.run(["media-ctl", "-V", "'ipu1_csi0':2 [fmt:"+cam_col_fmt+"/"+grab_res+"]"])

# set camera parameters via v4l2-ctl
subprocess.run(["v4l2-ctl", "-d", str(IPU1_CSI0_DEVICE), "-c", str(v4l2_ctl_param)])	# set v4l2 controls 

if cam_col_fmt == "SGRBG8_1X8":
   cap = cv2.VideoCapture("v4l2src device="+str(IPU1_CSI0_DEVICE)+" ! video/x-bayer, format=grbg, width="+str(cam_width)+", height="+str(cam_height)+" ! bayer2rgbneon ! videoconvert ! appsink", cv2.CAP_GSTREAMER)
else:
   cap = cv2.VideoCapture("v4l2src device="+str(IPU1_CSI0_DEVICE)+" ! video/x-raw,format=GRAY8,depth=8, width="+str(cam_width)+", height="+str(cam_height)+" ! appsink", cv2.CAP_GSTREAMER)

#cap = cv2.VideoCapture(IPU1_CSI0_DEVICE_NUMBER, cv2.CAP_V4L)
#cap = cv2.VideoCapture(0, cv2.CAP_V4L)
#cap = cv2.VideoCapture("v4l2src device=/dev/video0 ! video/x-bayer, format=grbg, width="+str(cam_width)+", height="+str(cam_height)+" ! bayer2rgbneon ! videoconvert ! appsink", cv2.CAP_GSTREAMER)

while(True):
    ret, frame = cap.read()

    cv2.imshow('phyCAM - Live Image',frame)
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break
 
cap.release()
cv2.destroyAllWindows()